# -*- coding: UTF-8 -*-

import optparse
import os
import sys
import tempfile
import time
import traceback

sys.path[0] = os.path.normpath('%s/../../..' % sys.path[0])
from common import config, logger, macros, statuses
from common.rest_state import RestState as rest_state
from common.statuses import unload as st
from artix.common import servicelock
from unload.node.cash import nodeGetter
from unload.service.reader import Reader


def unload(reader, node, log, workDirectory, request=None, converterName=None, shifts=None, dates=None,
        granularity='document', closedShifts=0, changedShifts=False, cashUnloadStateId=None, statusHost=None, isSendDebugStatus=True, unloadToDay=False):
    unloadNowFile = os.path.normpath('%s/locks/lockArtixUnloadNowForShop%s' % (tempfile.gettempdir(), node.id))
    srvLock = servicelock.ServiceLock(unloadNowFile, log)
    if srvLock.lock(False):
        try:
            flag_read = True
            flag_send = True
            if request and converterName:
                log.info(u'Выгрузка по запросу %s' % time.time())
                shifts = None
                dates = None
                try:
                    plugin = __import__('unload.converter.%s.%s' % (converterName, converterName), fromlist='q')
                    shifts, dates = plugin.parseFlag(request)
                except Exception as e:
                    statuses.setUnloadStatus(st.ERROR_IMPORT_PLUGIN)
                    log.error(e)
                    return False

                flag_read = reader.read(node, closedShifts, shifts, dates,
                        "document" if not shifts and not dates else "shift", False, workDirectory,
                        converterName=converterName, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
                log.info(u'Удаление файла-флага %s' % request)
                os.remove(request)
            else:
                flag_read = reader.read(node, closedShifts, shifts, dates, granularity, changedShifts, workDirectory,
                        converterName=converterName, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus, unloadToDay=unloadToDay)

            if os.path.exists(reader.getNodeStatusFileName(workDirectory, node.id, closedShifts == 1)):
                flag_send = reader.send(node, workDirectory, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus, closedShifts=closedShifts)

            if flag_read and flag_send:
                statuses.setUnloadStatus(st.UNLOAD_FINISH)

        except Exception as e:
            statuses.setUnloadStatus(st.ERROR_UNKNOWN)
            log.error(u'Ошибка %s' % str(e))
            log.error(u'Стек вызовов: %s' % traceback.format_exc())
            rest_state.sendSaleState(node, None, None, 'An error occurred during unload sales: %s' % str(e), u'Произошла ошибка при выгрузке продаж', rest_state.STATUS_ERROR, None, 
                                     cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
        finally:
            srvLock.unlock()
        return True
    else:
        statuses.setUnloadStatus(st.ERROR_ALREADY_RUN)
        log.info(u'Не удалось запустить процесс выгрузки. Выгрузка уже запущена...')
        return False


def mainRun(shifts, dates, granularity='document', closedShifts=0, changedShifts=False, converterName=None,
        dataPath=None, cashUnloadStateId=None, statusHost=None, isSendDebugStatus=True, deleteCurrentSales=False, unloadToDay=False):
    reader = Reader()
    node = nodeGetter.getNodes()[0]
    log = logger.getUnloadLogger("Node", node.id)
    log.info(u"Запуск процесса выгрузки продаж")

    if not config.isValid():
        statuses.setUnloadStatus(st.ERROR_READ_INI)
        log.error(u"Файл настроек %s не найден" % config.getConffiles())
        return False

    statuses.setUnloadStatus(st.PROGRAM_START)

    workDirectory = config.getString('main', 'workDirectory', '/linuxcash/cash/data/tmp/exchangers')
    reader.finishFlagPath = config.getString("UNLOAD", "pathUnloadFinishFlag", None)
    reader.processFlagPath = config.getString("UNLOAD", "pathUnloadProcessFlag", None)

    if deleteCurrentSales:
        reader.deleteCurrentSales(node, workDirectory)

    if converterName:
        if not any(x.name == converterName for x in node.cashes[0].converters):
            # Если конвертора из параметра нет в ноде, добавим его сами
            converter = nodeGetter.getConverter(converterName)
            if converter:
                node.cashes[0].converters.append(converter)
            else:
                statuses.setUnloadStatus(st.ERROR_OPTIONS)
                log.error("Не поддерживаемый тип конвертера %s" % converterName)
                return False

        for converter in node.cashes[0].converters:
            if converter.name == converterName and dataPath:
                converter.path = macros.applyMacros(dataPath, log, shopid=node.id, cashcode=node.cashes[0].code, cashid=node.cashes[0].cashId)

    return unload(reader, node, log, workDirectory, shifts=shifts, dates=dates, granularity=granularity,
                  closedShifts=closedShifts, changedShifts=changedShifts, converterName=converterName,
                  cashUnloadStateId=cashUnloadStateId, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus, unloadToDay=unloadToDay)


def main(argv):
    parser = optparse.OptionParser()

    parser.add_option("-s", "--shifts", dest="shifts", help=u"@ - разделитель периода смен")
    parser.add_option("-d", "--dates", dest="dates", help=u"@ - разделитель периода дат")
    parser.add_option("-g", "--granularity", dest="granularity", default="document", help=u"детализация по сменам или документам. Значения: shift, document [default: %default]")
    parser.add_option("-c", "--closedShifts", dest="closedShifts", default=0, type='int',
                      help=u"Выгрузка только открытых или закрытых смен. Значения: 0 - закрытые, 1 - открытые, 2 - открытые и закрытые  [default: %default]")
    parser.add_option("-C", "--changedShifts", action="store_true", dest="changedShifts", default=False, help=u"Выгрузка документов только для измененных смен")
    parser.add_option("-v", "--converter", dest="converterName", help=u"Тип конвертора: aif, atol-astor, crystal, ns2000, oracle, shtrih, smag")
    parser.add_option("-p", "--path", dest="dataPath", help=u"Путь куда выгружаются файлы продаж")
    parser.add_option("-u", "--cashUnloadStateId", dest="cashUnloadStateId", help=u"Идентификатор статуса выгрузки продаж")
    parser.add_option("-r", "--statusHost", dest="statusHost", help=u"Хост и порт REST сервера разделенный :")
    parser.add_option("-D", "--debugStatus", action="store_true", dest="isSendDebugStatus", default=True, help=u"Отправлять статусы выгрузки продаж на сервер в debug режиме")
    parser.add_option("-o", "--deleteCurrentSales", action="store_true", dest="deleteCurrentSales", help=u"Удаление файлов текущих продаж. Используется после закрытии смены")
    parser.add_option("-t", "--today", action="store_true", dest="today", default=False, help=u"Выгрузить документы за смены, открытые за текущий день")

    (options, args) = parser.parse_args(argv)

    result = mainRun(options.shifts, options.dates, options.granularity, options.closedShifts,
            options.changedShifts, options.converterName, options.dataPath, options.cashUnloadStateId,
            options.statusHost, options.isSendDebugStatus, options.deleteCurrentSales, options.today)
    if not result:
        os._exit(1)  # выход без выполнения atexit


if __name__ == '__main__':
    main(sys.argv[1:])
